
''' -----------------------------------------------------------------------------
''' Project	 : VBCommenter
''' Class	 : Configuration
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' Handle saving and loading of config settings for the application
''' </summary>
''' <remarks>
''' Uses Singleton design pattern. Only a single instance of this object
''' is ever required, so the GetInstance() method is used to create or access
''' the object.
''' <para>
''' The data is persisted through an implementation of the IConfigProvider interface.
''' </para>
''' </remarks>
''' -----------------------------------------------------------------------------
Public Class Configuration

	Private Const keyCreateXmlFiles As String = "CreateXmlFiles"
	Private Const keyInsertXMLComments As String = "InsertXMLCommentsInSource"
    Private Const keyAddHistoryTags As String = "AddHistoryTags"
    Private Const keyUseCustomXMLStubTemplate As String = "UseCustomXMLStubTemplate"
    Private Const keyCustomXMLStubTemplatePath As String = "CustomXMLStubTemplatePath"

	Private Const keyStripLinesOnlyContaining As String = "StripLinesOnlyContaining"
	Private Const keyXMLCommentPrefix As String = "XMLCommentPrefix"

	Private Shared _Instance As Configuration = Nothing
	Private Shared mConfigImp As IConfigProvider

	Protected Sub New()
		mConfigImp = New RegistryConfigImp
	End Sub


	''' -----------------------------------------------------------------------------
	''' <summary>
	''' Returns a reference to the Configuration object
	''' </summary>
	''' <returns>a Configuration object</returns>
	''' <remarks>
	''' 
	''' </remarks>
	''' <history>
	''' 	[David Gardiner]	16 Sep 2003	Created
	''' </history>
	''' -----------------------------------------------------------------------------
	Public Shared Function GetInstance() As Configuration
		If _Instance Is Nothing Then
			_Instance = New Configuration
		End If

		Return _Instance

	End Function


	<ComponentModel.Category("General"), _
	ComponentModel.Description("This setting controls the prefix comments should have in order to be recognised as XML comments. (the prefix includes the start-comment ' symbol"), _
	ComponentModel.DefaultValue("'''")> _
	Public Property XMLCommentPrefix() As String
		Get
			Return mConfigImp.GetSetting(keyXMLCommentPrefix, "'''")
		End Get
		Set(ByVal sValue As String)
			mConfigImp.SetSetting(keyXMLCommentPrefix, sValue)
		End Set
	End Property

	<ComponentModel.Category("General"), _
	ComponentModel.Description("Remove any lines in the XML comments that only contains these characters"), _
	ComponentModel.DefaultValue("- ")> _
	Public Property StripLinesOnlyContaining() As String
		Get
			Return mConfigImp.GetSetting(keyStripLinesOnlyContaining, "'/*-+")
		End Get
		Set(ByVal sValue As String)
			mConfigImp.SetSetting(keyStripLinesOnlyContaining, sValue)
		End Set
	End Property

	<ComponentModel.Category("General"), _
	ComponentModel.Description("Create a .xml file for each project upon rebuild.")> _
	Public Property CreateXMLFiles() As Boolean
		Get
			Return mConfigImp.GetSetting(keyCreateXmlFiles, True)
		End Get
		Set(ByVal Value As Boolean)
			mConfigImp.SetSetting(keyCreateXmlFiles, Value)

		End Set
	End Property

	<ComponentModel.Category("General"), _
	ComponentModel.Description("Insert XML comments into the source code")> _
	Public Property InsertXMLComments() As Boolean
		Get
			Return mConfigImp.GetSetting(keyInsertXMLComments, True)
		End Get
		Set(ByVal Value As Boolean)
			mConfigImp.SetSetting(keyInsertXMLComments, Value)

		End Set
	End Property

	<ComponentModel.Category("General"), _
	ComponentModel.Description("Include <history> tags in the XML comments inserted into the source code")> _
	Public Property AddHistoryTags() As Boolean
		Get
			Return mConfigImp.GetSetting(keyAddHistoryTags, True)
		End Get
		Set(ByVal Value As Boolean)
			mConfigImp.SetSetting(keyAddHistoryTags, Value)

		End Set
	End Property


    <ComponentModel.Category("General"), _
    ComponentModel.Description("Should we use a custom XML stub template?")> _
    Public Property UseCustomXMLStubTemplate() As Boolean
        Get
            Return mConfigImp.GetSetting(keyUseCustomXMLStubTemplate, True)
        End Get
        Set(ByVal Value As Boolean)
            mConfigImp.SetSetting(keyUseCustomXMLStubTemplate, Value)
        End Set
    End Property

    <ComponentModel.Category("General"), _
    ComponentModel.Description("Location of the custom XML stub template?")> _
    Public Property CustomXMLStubTemplatePath() As Boolean
        Get
            Return mConfigImp.GetSetting(keyCustomXMLStubTemplatePath, True)
        End Get
        Set(ByVal Value As Boolean)
            mConfigImp.SetSetting(keyCustomXMLStubTemplatePath, Value)
        End Set
    End Property

End Class

''' -----------------------------------------------------------------------------
''' Project	 : VBCommenter
''' Interface	 : IConfigProvider
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' 
''' </summary>
''' <remarks>
''' </remarks>
''' <history>
''' 	[David Gardiner]	16/09/2003	Created
''' </history>
''' -----------------------------------------------------------------------------
Interface IConfigProvider

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <param name="default"></param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[David Gardiner]	16/09/2003	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Function GetSetting(ByVal name As String, ByVal defaultValue As String) As String


    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <param name="default"></param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[David Gardiner]	16/09/2003	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Function GetSetting(ByVal name As String, ByVal defaultValue As Boolean) As Boolean

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[David Gardiner]	16/09/2003	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Sub SetSetting(ByVal name As String, ByRef value As String)

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="name"></param>
    ''' <param name="value"></param>
    ''' <remarks>
    ''' </remarks>
    ''' <history>
    ''' 	[David Gardiner]	16/09/2003	Created
    ''' </history>
    ''' -----------------------------------------------------------------------------
    Sub SetSetting(ByVal name As String, ByVal value As Boolean)
End Interface

''' -----------------------------------------------------------------------------
''' Project	 : VBCommenter
''' Class	 : XmlDocumentConfigImpl
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' Implements IConfigProvider using an XML document for persistant storage
''' </summary>
''' <remarks>
''' </remarks>
''' <history>
''' 	[David Gardiner]	16/09/2003	Created
''' </history>
''' -----------------------------------------------------------------------------
Class XmlDocumentConfigImp
    Implements IConfigProvider

    Private moSettingsCache As Hashtable

    Public Sub New()
        moSettingsCache = New Hashtable

    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Return the full path to the config file.
    ''' </summary>
    ''' <returns>a String</returns>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Private Function GetConfigFilename() As String
        Return IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), "VBCommentorConfig.xml")
    End Function



    Public Overloads Function GetSetting(ByVal name As String, ByVal defaultValue As Boolean) As Boolean Implements IConfigProvider.GetSetting
        Dim sTmpValue As String
        Dim sDefaultValue As String

        If defaultValue Then
            sDefaultValue = "1"
        Else
            sDefaultValue = "0"
        End If

        sTmpValue = GetSetting(name, sDefaultValue)
        If sTmpValue.Trim() = "1" Then
            Return True
        Else
            Return False
        End If

    End Function

    Public Overloads Function GetSetting(ByVal name As String, ByVal defaultValue As String) As String Implements IConfigProvider.GetSetting

        If Not moSettingsCache.ContainsKey(name) Then
            moSettingsCache.Add(name, defaultValue)
            Return defaultValue
        Else
            Return CStr(moSettingsCache.Item(name))
        End If
    End Function

    Public Overloads Sub SetSetting(ByVal name As String, ByVal value As Boolean) Implements IConfigProvider.SetSetting
        Dim sValue As String
        If value Then
            sValue = "1"
        Else
            sValue = "0"
        End If

        SetSetting(name, sValue)

    End Sub

    Public Overloads Sub SetSetting(ByVal name As String, ByRef value As String) Implements IConfigProvider.SetSetting

        If moSettingsCache.ContainsKey(name) Then
            moSettingsCache.Item(name) = value
        Else
            moSettingsCache.Add(name, value)
        End If

    End Sub
End Class

''' -----------------------------------------------------------------------------
''' Project	 : VBCommenter
''' Class	 : RegistryConfigImpl
''' 
''' -----------------------------------------------------------------------------
''' <summary>
''' Implements the IConfigProvider interface using the Windows Registry to persist data.
''' </summary>
''' <remarks>
''' Uses a StringDictionary to cache values
''' </remarks>
''' <history>
''' 	[David Gardiner]	16/09/2003	Created
''' </history>
''' -----------------------------------------------------------------------------
Class RegistryConfigImp
    Implements IConfigProvider

    Private Const RegistrySubKeyName As String = "Software\PowerToys for Visual Studio .NET 2003\VBCommenter"

    Private moSettingsCache As System.Collections.Specialized.StringDictionary

    Dim rk As Microsoft.Win32.RegistryKey

    Public Sub New()
        moSettingsCache = New System.Collections.Specialized.StringDictionary
        rk = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(RegistrySubKeyName, True)

        ' create if it did not exist.
        If rk Is Nothing Then
            rk = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(RegistrySubKeyName)

        End If

    End Sub

    Public Overloads Function GetSetting(ByVal name As String, ByVal defaultValue As Boolean) As Boolean Implements IConfigProvider.GetSetting
        Dim value As Boolean
        If Not moSettingsCache.ContainsKey(name) Then

            ' not in cache, so look in registry
            Dim obj As Object = rk.GetValue(name)

            If obj Is Nothing Then
                ' save default to registry
                rk.SetValue(name, System.Convert.ToInt32(defaultValue))

                value = defaultValue

            Else
                value = CBool(obj)
            End If

            ' save to cache for next time
            moSettingsCache.Add(name, value.ToString())
        Else
            ' get from cache
            value = CBool(moSettingsCache.Item(name))
        End If

        Return value
    End Function

    Public Overloads Function GetSetting(ByVal name As String, ByVal defaultValue As String) As String Implements IConfigProvider.GetSetting
        Dim value As String

        If Not moSettingsCache.ContainsKey(name) Then

            ' not in cache, so look in registry
            Dim obj As Object = rk.GetValue(name)

            If obj Is Nothing Then
                ' save default to registry
                rk.SetValue(name, defaultValue)

                value = defaultValue

            Else
                value = CStr(obj)
            End If

            ' save to cache for next time
            moSettingsCache.Add(name, value)
        Else
            ' get from cache
            value = moSettingsCache.Item(name)
        End If

        Return value
    End Function

    Public Overloads Sub SetSetting(ByVal name As String, ByVal value As Boolean) Implements IConfigProvider.SetSetting

        ' save to cache
        moSettingsCache.Item(name) = value.ToString()

        ' and to registry
        rk.SetValue(name, Convert.ToInt32(value))
    End Sub

    Public Overloads Sub SetSetting(ByVal name As String, ByRef value As String) Implements IConfigProvider.SetSetting

        ' save to cache
        moSettingsCache.Item(name) = value

        ' and to registry
        rk.SetValue(name, value)
    End Sub
End Class