Option Strict On
Option Explicit On 

''' <summary>
''' Describes how to move the cursor
''' </summary>
Public Enum ColumnMoveMode
    ''' <summary>
    ''' Start at the absoloute start of the line
    ''' </summary>
    icStartAtAbsoluteStart = 1
    ''' <summary>
    ''' Start at the current indentation level on the line
    ''' </summary>
    icStartAtTextStart = 2
    ''' <summary>
    ''' Do not move the cursor
    ''' </summary>
    icDoNotMove = 3
    ''' <summary>
    ''' Start from the current position
    ''' </summary>
    icStartFromCurPos = 4
    ''' <summary>
    ''' Move cursor to the end of the line
    ''' </summary>
    icMoveToEndOfLine = 5
End Enum

Public Class TextInsertion

    Private moEditPoint As EnvDTE.EditPoint
    Private msInsertText As String
    Private mlMoveLinesDownAfterInsert As Integer
    Private mlMoveColumnsAfterInsert As Integer
    Private miColumnMoveModeAfterInsert As ColumnMoveMode
    Private miSoftNewlineCount As Integer
    Private mbGenerateSoftLinesOnNegativeStart As Boolean

    Private mlMoveLinesDownPreInsert As Integer
    Private mlMoveColumnsPreInsert As Integer
    Private miColumnMoveModePreInsert As ColumnMoveMode

    Public Property SoftNewlineCount() As Integer
        Get
            Return miSoftNewlineCount
        End Get
        Set(ByVal Value As Integer)
            miSoftNewlineCount = Value
        End Set
    End Property

    Public Property GenerateSoftLinesOnNegativeStart() As Boolean
        Get
            Return mbGenerateSoftLinesOnNegativeStart
        End Get
        Set(ByVal Value As Boolean)
            mbGenerateSoftLinesOnNegativeStart = Value
        End Set
    End Property

    Public Property ColumnMoveModeAfterInsert() As ColumnMoveMode
        Get
            Return miColumnMoveModeAfterInsert
        End Get
        Set(ByVal Value As ColumnMoveMode)
            miColumnMoveModeAfterInsert = Value
        End Set
    End Property

    Public Property ColumnMoveModePreInsert() As ColumnMoveMode
        Get
            Return miColumnMoveModePreInsert
        End Get
        Set(ByVal Value As ColumnMoveMode)
            miColumnMoveModePreInsert = Value
        End Set
    End Property

    Public Property MoveColumnsAfterInsert() As Integer
        Get
            Return mlMoveColumnsAfterInsert
        End Get
        Set(ByVal Value As Integer)
            mlMoveColumnsAfterInsert = Value
        End Set
    End Property

    Public Property MoveColumnsPreInsert() As Integer
        Get
            Return mlMoveColumnsPreInsert
        End Get
        Set(ByVal Value As Integer)
            mlMoveColumnsPreInsert = Value
        End Set
    End Property

    Public Property MoveLinesDownAfterInsert() As Integer
        Get
            Return mlMoveLinesDownAfterInsert
        End Get
        Set(ByVal Value As Integer)
            mlMoveLinesDownAfterInsert = Value
        End Set
    End Property

    Public Property MoveLinesDownPreInsert() As Integer
        Get
            Return mlMoveLinesDownPreInsert
        End Get
        Set(ByVal Value As Integer)
            mlMoveLinesDownPreInsert = Value

            If mlMoveLinesDownPreInsert < 0 Then
                If mbGenerateSoftLinesOnNegativeStart Then
                    miSoftNewlineCount = (mlMoveLinesDownPreInsert * -1)
                End If
            End If
        End Set
    End Property

    Sub New(ByVal poEditPoint As EnvDTE.EditPoint)
        moEditPoint = poEditPoint
        Reset()
    End Sub

    Public ReadOnly Property HasAnythingToAdd() As Boolean
        Get
            Return (msInsertText.Length > 0)
        End Get
    End Property

    Public Sub PerformInsertion()

        Dim iSoftLinesLeft As Integer
        Dim iCurLine As Integer
        Dim aLines() As String
        Dim sTmp As String

        Dim oPreEditPoint As EnvDTE.EditPoint
        Dim oSubEditPoint As EnvDTE.EditPoint

		' start an Undo context
		Const UndoContextName As String = "PerformInsertion"
        'undo stuff seemed to be causing a memory leak, removed it and Undo still works
        'moEditPoint.DTE.UndoContext.Open(UndoContextName)

        If mlMoveLinesDownPreInsert <> 0 Then
            If mlMoveLinesDownPreInsert > 0 Then
                moEditPoint.LineDown(mlMoveLinesDownPreInsert)
            Else
                moEditPoint.LineUp(mlMoveLinesDownPreInsert * -1)
            End If
        End If

        Select Case miColumnMoveModePreInsert
            Case ColumnMoveMode.icDoNotMove
                ' Do nothing
            Case ColumnMoveMode.icMoveToEndOfLine
                moEditPoint.EndOfLine()
            Case ColumnMoveMode.icStartAtAbsoluteStart
                moEditPoint.StartOfLine()
            Case ColumnMoveMode.icStartAtTextStart
                moEditPoint.StartOfLine()
            Case ColumnMoveMode.icStartFromCurPos
                ' Do nothing
        End Select

        If (miColumnMoveModePreInsert <> ColumnMoveMode.icDoNotMove) And _
           (miColumnMoveModePreInsert <> ColumnMoveMode.icMoveToEndOfLine) Then
            If mlMoveColumnsPreInsert <> 0 Then
                moEditPoint.MoveToLineAndOffset(moEditPoint.Line, moEditPoint.LineCharOffset + mlMoveColumnsPreInsert)
            End If
        End If

        oPreEditPoint = moEditPoint.CreateEditPoint()
        oPreEditPoint.MoveToLineAndOffset(moEditPoint.Line, moEditPoint.LineCharOffset)

        If miSoftNewlineCount > 0 Then
            sTmp = msInsertText.Replace(vbCrLf, Chr(13).ToString)
            aLines = sTmp.Split(Chr(13))

            iSoftLinesLeft = miSoftNewlineCount
            For iCurLine = 0 To aLines.Length - 1
                If iCurLine > 0 Then
                    If iSoftLinesLeft > 0 Then
                        moEditPoint.LineDown()
                        iSoftLinesLeft -= 1
                    Else
                        moEditPoint.Insert(vbCrLf)
                        'moEditPoint.SmartFormat()
                        'moEditPoint.NewLine()
                    End If
                End If
                moEditPoint.Insert(aLines(iCurLine))
            Next

        Else
            moEditPoint.Insert(msInsertText)
        End If

        moEditPoint.EndOfLine()
        moEditPoint.Parent().Selection().MoveToPoint(moEditPoint)

        oSubEditPoint = moEditPoint.CreateEditPoint()
        oSubEditPoint.MoveToLineAndOffset(moEditPoint.Line, moEditPoint.LineCharOffset)

        oPreEditPoint.SmartFormat(oSubEditPoint)

        If mlMoveLinesDownAfterInsert <> 0 Then
            If mlMoveLinesDownAfterInsert > 0 Then
                moEditPoint.Parent().Selection().LineDown(False, mlMoveLinesDownAfterInsert)
            Else
                moEditPoint.Parent().Selection().LineUp(False, mlMoveLinesDownAfterInsert * -1)
            End If
        End If

        Select Case miColumnMoveModeAfterInsert
            Case ColumnMoveMode.icDoNotMove
                ' Do nothing
            Case ColumnMoveMode.icMoveToEndOfLine
                moEditPoint.Parent().Selection().EndOfLine()
            Case ColumnMoveMode.icStartAtAbsoluteStart
                moEditPoint.Parent().Selection().StartOfLine()
            Case ColumnMoveMode.icStartAtTextStart
                moEditPoint.Parent().Selection().StartOfLine(EnvDTE.vsStartOfLineOptions.vsStartOfLineOptionsFirstText)
            Case ColumnMoveMode.icStartFromCurPos
                ' Do nothing
        End Select

        If (miColumnMoveModeAfterInsert <> ColumnMoveMode.icDoNotMove) And _
           (miColumnMoveModeAfterInsert <> ColumnMoveMode.icMoveToEndOfLine) Then
            If mlMoveColumnsAfterInsert <> 0 Then
                moEditPoint.Parent().Selection().MoveToLineAndOffset(moEditPoint.Parent().Selection().ActivePoint.Line, moEditPoint.Parent().Selection().ActivePoint.LineCharOffset + mlMoveColumnsAfterInsert)
            End If
        End If

		' Close Undo context
        'moEditPoint.DTE.UndoContext.Close()
    End Sub

    Public Sub Reset()
        msInsertText = ""
        mlMoveLinesDownAfterInsert = 0
        mlMoveColumnsAfterInsert = 0
        miSoftNewlineCount = 0
        mbGenerateSoftLinesOnNegativeStart = True
        miColumnMoveModeAfterInsert = ColumnMoveMode.icDoNotMove

        mlMoveLinesDownPreInsert = 0
        mlMoveColumnsPreInsert = 0
        miColumnMoveModePreInsert = ColumnMoveMode.icDoNotMove
    End Sub

    Public Sub AddLine(ByVal psLineContent As String)
        If msInsertText.Length > 0 Then
            msInsertText += vbCrLf
        End If
        msInsertText += psLineContent
    End Sub

End Class
